module Docker
  module Porcelain
    module Image
      module ClassMethods
        # Get image specified by id
        # @param id [String] identifier of the image to get
        # @return [Docker::Image, nil] image or nil if not found
        def [] id
          Docker::Image.get id
        rescue Docker::Error::NotFoundError
          nil
        end

        # Get all images from the specified repository (eg. as in 'repository:tag')
        # @param repo [String] repository to look for
        # @return [Array<Docker::Image>] images
        def repository repo
          Docker::Image.all 'filter' => repo
        end
      end

      # @return [Array<String>] repository tags of the image
      def tags
        # Image can have incomplete info, refresh then
        refresh! unless info['RepoTags']

        # docker returns '<none>:<none>' for an untagged repo
        info['RepoTags'] - ['<none>:<none>']
      end

      # @return [Docker::Image] the parent image
      # @return [nil] if the image is +FROM scratch+
      def parent
        return nil if (parent = info['Parent']).empty?
        Docker::Porcelain::Image[parent]
      end

      # Tag the image
      # @overload tag(repo, tag, opts)
      #   @param [String] repo the repository to tag in
      #   @param [String] tag the new tag
      #   @option opts [Boolean] force whether to replace the tag if it
      #     already exists
      # @overload tag(repo_tag, opts)
      #   @param [String] repo_tag the repository to tag in with optional tag;
      #     in 'repo:tag' or 'repo' format
      #   @option opts [Boolean] force whether to replace the tag if it
      #     already exists
      # @overload tag(opts)
      #   @option opts [String] repo the repository to tag in
      #   @option opts [String] tag the new tag
      #   @option opts [Boolean] force whether to replace the tag if it
      #     already exists
      # @return [void]
      def tag *args
        strargs, rest = args.partition { |arg| arg.respond_to? :to_str }

        # rest should be [opts] or empty
        rest.length < 2 or
            fail ArgumentError, 'expected args: [repo [tag]] [options]'
        opts = rest.first || {}

        super opts.merge Image.repo_tag_to_hash *strargs
      end

      extend ClassMethods

      private

      def self.repo_tag_to_hash repo = nil, tag = nil
        return {} unless repo
        repo, tag = repo.split ':' unless tag
        { 'repo' => repo, 'tag' => tag }
      end
    end
  end
end
